﻿/*
 * This is part of PosXp
 * Copyright (C) 2006-2013  Alberto Rivelli
 * 
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. 
 */


using System;
using System.Runtime.InteropServices;
using System.Security.Principal;
using Microsoft.Interop.Security.AzRoles;


namespace POS.Framework
{

    internal class AzmanPrincipal : IPrincipal
    {

        private WindowsIdentity m_identity;
        private AzAuthorizationStore AzStore = new AzAuthorizationStore();
        private IAzApplication Azapp;
        private string[] m_cache;

        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        //Struttura operazione/permesso
        //\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
        private struct OpItem
        {
            internal int id;
            internal string name;

            public int names
            {
                get
                {
                    return names;
                }
            }

        }

        public AzmanPrincipal(WindowsIdentity identity, string filepath)
        {
            if (identity == null)
            {
                throw new ArgumentNullException("Identity");
            }
            this.m_identity = identity;

            try
            {

                AzStore.Initialize(0, "msxml://" + filepath, null);

                Azapp = AzStore.OpenApplication("Posxp", null);

                //Create cache of operations in the store
                m_cache = _cacheOps(Azapp.Operations);

            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message);
            }

        }

        IIdentity IPrincipal.Identity
        {
            get
            {
                return m_identity;
            }
        }

        internal string[] CachedOps
        {
            get
            {
                return m_cache;
            }
        }

        bool IPrincipal.IsInRole(string role)
        {

            foreach (string dd in GetRoles)
            {
                if (dd == role) return true;
            }

            return false;

        }



        //proprieta Roles ritorna i ruoli non cachati
        internal string[] GetRoles
        {
            get
            {
                string[] a = new string[Azapp.Roles.Count];
                int i = 0;

                foreach (IAzRole rr in Azapp.Roles)
                {
                    a[i] = rr.Name;
                    i++;
                }

                return a;
            }
        }

        //proprieta Operations ritorna le operazioni cachate
        internal object[] GetOperations(string role)
        {

            IAzTask sa;
            IAzRole azrole;
            object[] opcoll = null;

            if (role == null)
            {
                throw new ArgumentException();
            }
            else
            {
                try
                {
                    azrole = Azapp.OpenRole(role, null);

                    sa = Azapp.OpenTask(((object[])azrole.Tasks)[0].ToString(), null);

                }
                catch (COMException ex)
                {
                    throw new AzMethodException(ex, "IAzClientContext.GetOperations");
                }

                opcoll = (object[])sa.Operations;

            }

            return opcoll;

        }

        internal void AddOperation(string role, string operation)
        {

            IAzRole azrole;
            IAzTask sa;

            if (role == null || operation == null)
            {
                throw new ArgumentException();
            }
            else
            {
                try
                {

                    azrole = Azapp.OpenRole(role, null);

                    sa = Azapp.OpenTask(((object[])azrole.Tasks)[0].ToString(), null);

                    sa.AddOperation(operation, null);

                    sa.Submit(0, null);

                }
                catch (COMException ex)
                {
                    if (ex.ErrorCode == -2147024713)
                    {
                        return;
                    }
                    throw new AzMethodException(ex, "IAzRole.AddOperation");
                }

            }

        }

        internal void RemoveOperation(string role, string operation)
        {

            IAzRole azrole;
            IAzTask sa;

            if (role == null || operation == null)
            {
                throw new ArgumentException();
            }
            else
            {
                try
                {

                    azrole = Azapp.OpenRole(role, null);

                    sa = Azapp.OpenTask(((object[])azrole.Tasks)[0].ToString(), null);

                    sa.DeleteOperation(operation, null);

                    sa.Submit(0, null);

                }
                catch (COMException ex)
                {
                    throw new AzMethodException(ex, "IAzRole.RemoveOperation");
                }

            }

        }

        //proprieta Operations ritorna le operazioni cachate
        internal object[] GetUsers(string role)
        {

            IAzRole azrole;
            object[] userscoll = null;

            if (role == null)
            {
                throw new ArgumentException();
            }
            else
            {
                try
                {
                    azrole = Azapp.OpenRole(role, null);

                }
                catch (COMException ex)
                {
                    throw new AzMethodException(ex, "IAzClientContext.GetUsers");
                }

                userscoll = (object[])azrole.MembersName;

            }

            return userscoll;

        }

        internal void AddUser(string role, string group)
        {

            IAzRole azrole;

            if (role == null || group == null)
            {
                throw new ArgumentException();
            }
            else
            {
                try
                {

                    azrole = Azapp.OpenRole(role, null);

                    azrole.AddMemberName(group, null);

                    azrole.Submit(0, null);

                }
                catch (COMException ex)
                {
                    // group already bind to role
                    if (ex.ErrorCode == -2147024713)
                    {
                        return;
                    }
                    throw new AzMethodException(ex, "IAzRole.AddUser");
                }

            }

        }

        internal bool HasAccess(string opname)
        {

            IAzClientContext AzContext;
            object[] operations;
            object[] scopes = { "" };
            object[] result;

            try
            {

                IAzOperation op = Azapp.OpenOperation(opname, null);

                operations = new Object[] { (int)(op.OperationID) };

                Marshal.ReleaseComObject(op);

            }
            catch (COMException ex)
            {

                //ex.ErrorCode = AzManConstants.INT_NO_SUCH_OBJECT Then
                throw new AzMethodException(ex, "AzManPrincipal.OperationNotFound");

            }

            AzContext = Azapp.InitializeClientContextFromToken((ulong)(m_identity.Token), null);

            result = (object[])AzContext.AccessCheck("Prova", scopes, operations, null, null, null, null, null);

            if ((int)result[0] == 0)
            {
                return true;
            }
            else
            {
                return false;
            }

        }

        internal bool HasAccess(int opid)
        {

            IAzClientContext AzContext;
            object[] operations;
            object[] scopes = { "" };
            object[] result;

            try
            {

                operations = new Object[] { (int)(opid) };

                AzContext = Azapp.InitializeClientContextFromToken((ulong)(m_identity.Token), null);

                result = (object[])AzContext.AccessCheck("Prova", scopes, operations, null, null, null, null, null);

            }
            catch (COMException ex)
            {

                //ex.ErrorCode = AzManConstants.INT_NO_SUCH_OBJECT Then
                throw new AzMethodException(ex, "AzManPrincipal.OperationNotFound");

            }

            if ((int)result[0] == 0)
            {
                return true;
            }
            else
            {
                return false;
            }

        }

        private string[] _cacheOps(IAzOperations ops)
        {

            string[] cache = new string[ops.Count];
            int i = 0;

            foreach (IAzOperation op in ops)
            {
                cache[i] = op.Name;
                i += 1;
            }

            return cache;

        }

    }

    internal class AzMethodException : ApplicationException
    {

        public readonly int errorcode;

        public AzMethodException(COMException x, string methodName)
            : base(String.Format(x.Message + "The method called was {0}, and the error code was {1}.", methodName, x.ErrorCode))
        {

            this.errorcode = x.ErrorCode;

        }

    }

}
